#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <set>

typedef long long llong;
const int MAXN = 150000 + 10;
const int INF  = 1e9;

int n, m;
int c[MAXN];
int d[MAXN];
int perm[MAXN];
std::vector <int> g[MAXN];

struct DSU
{
    int par[MAXN];
    std::set <int> vals[MAXN];

    void reset()
    {
        std::iota(par + 1, par + 1 + n, 1);
        for (int i = 1 ; i <= n ; ++i)
        {
            vals[i].clear();
            vals[i].insert(c[i]);
        }
    }

    int find(int node)
    {
        if (par[node] == node) return node;
        return par[node] = find(par[node]);
    }

    void connect(int u, int v)
    {
        u = find(u);
        v = find(v);
        if (u == v)
        {
            return;
        }

        if (vals[u].size() > vals[v].size())
        {
            std::swap(u, v);
        }

        for (const int &val : vals[u])
        {
            vals[v].insert(val);
        }

        par[u] = v;
    }

    bool in(int u, int val)
    {
        return vals[find(u)].count(val);
    }
};

DSU dsu;
std::vector <int> byD[MAXN];
void reset()
{
    for (int i = 1 ; i <= n ; ++i)
    {
        g[i].clear();
        byD[i].clear();
    }
}

bool vis[MAXN];
bool dfs(int node, int val)
{
    if (c[node] == val)
    {
        return true;
    }

    vis[node] = true;
    for (const int &u : g[node])
    {
        if (!vis[u] && d[u] <= val && val <= c[u])
        {
            if (dfs(u, val))
            {
                return true;
            }
        }
    }

    return false;
}

void solve()
{
    for (int i = 1 ; i <= n ; ++i)
    {
        if (d[i] > c[i])
        {
            std::cout << 0 << '\n';
            return;
        }
    }

    std::iota(perm + 1, perm + 1 + n, 1);
    std::sort(perm + 1, perm + 1 + n, [&](int x, int y)
    {
        return d[x] < d[y];
    });

    for (int i = 1 ; i <= n ; ++i)
    {
        byD[d[i]].push_back(i);
    }

    dsu.reset();
    int ptrLeft = 1;
    int ptrRight = 1;

    int answered = 0;
    while (ptrRight <= n)
    {
        ptrLeft = ptrRight;
        while (ptrRight <= n && d[perm[ptrRight]] == d[perm[ptrLeft]])
        {
            for (const int &u : g[perm[ptrRight]])
            {
                if (d[u] <= d[perm[ptrRight]])
                {
                    // std::cout << "connect: " << u << ' ' << perm[ptrRight] << '\n';
                    dsu.connect(u, perm[ptrRight]);
                }
            }

            ptrRight++;
        }

        for (const int &u : byD[d[perm[ptrLeft]]])
        {
            std::fill(vis + 1, vis + 1 + n, 0);
            if (!dfs(u, d[u]))
            {
                std::cout << 0 << '\n';
                return;
            }

            // if (!dsu.in(u, d[u]))
            // {
            //     // std::cout << "bad: " << u << ' ' << d[u] << '\n';
            //     std::cout << 0 << '\n';
            //     return;
            // }

            answered++;
        }
    }

    if (answered < n)
    {
        std::cout << 0 << '\n';
        return;
    }

    std::cout << 1 << '\n';
}

void input()
{
    std::cin >> n >> m;
    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> c[i];
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        std::cin >> d[i];
    }

    for (int i = 1 ; i <= m ; ++i)
    {   
        int u, v;
        std::cin >> u >> v;
        g[u].push_back(v);
        g[v].push_back(u);
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int tests;
int main()
{
    fastIOI();
    std::cin >> tests;

    while (tests--)
    {
        reset();
        input();
        solve();
    }

    return 0;
}