#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>

typedef long long llong;
const int MAXN = 100000 + 10;
const int MOD = 1e9 + 7;

int n, d;
struct Value
{
    int l, w;
    void fix()
    {
        if (l >= MOD) l -= MOD;
        if (w >= MOD) w -= MOD;
    }

    friend Value operator + (const Value &left, const Value &right)
    {
        Value result;
        result.l = left.l + right.l;
        result.w = left.w + right.w;
        result.fix();
        return result;
    }

    friend Value operator - (const Value &left, const Value &right)
    {
        Value result;
        result.l = left.l + MOD - right.l;
        result.w = left.w + MOD - right.w;
        result.fix();
        return result;
    }
};

int sz[MAXN];
bool win[MAXN];
Value dpW[MAXN];
Value dpL[MAXN];
Value coefsL, coefsW;
std::vector <int> g[MAXN];

void dfs(int node, int par)
{
    int cnt = 0;
    int v = 0;

    sz[node] = 1;
    win[node] = false;
    for (const int &u : g[node])
    {
        if (u == par)
        {
            continue;
        }

        dfs(u, node);
        sz[node] += sz[u];
        win[node] |= !win[u];
        cnt += !win[u];

        if (!win[u])
        {
            v = u;
        }
    }

    if (cnt >= 2)
    {
        dpL[node] = {0, 0};
    } else if (cnt == 1)
    {
        dpL[node] = dpW[v];
    } else
    {
        dpL[node] = {0, 0};
        dpL[node].w = 1;
        for (const int &u : g[node])
        {
            if (u == par)
            {
                continue;
            }

            dpL[node] = dpL[node] + dpW[u];
        }
    }

    dpW[node].w = sz[node];
    dpW[node].l = sz[node];
    dpW[node] = dpW[node] - dpL[node];
    // std::cout << "dfs: " << node << ' ' << win[node] << ": " << dpL[node].l << ' ' << dpL[node].w << ", " << dpW[node].l << ' ' << dpW[node].w << '\n';
}

void solve()
{   
    // dfs(1, 0);
    // return;
    int l = 0;
    int w = 0;
    for (int i = 1 ; i <= n ; ++i)
    {
        dfs(i, 0);
        w += win[i];
        l += !win[i];
        coefsL = coefsL + dpL[i];
        coefsW = coefsW + dpW[i];
    }

    for (int i = 1 ; i < d ; ++i)
    {
        int newL, newW;
        newL = (1LL * coefsL.l * l + 1LL * coefsL.w * w) % MOD;
        newW = (1LL * coefsW.l * l + 1LL * coefsW.w * w) % MOD;
        l = newL; w = newW;
    }

    dfs(1, 0);
    int ans = (1LL * dpW[1].l * l + 1LL * dpW[1].w * w) % MOD;
    std::cout << ans << '\n';
}

void input()
{
    std::cin >> n >> d;
    for (int i = 1 ; i < n ; ++i)
    {
        int u, v;
        std::cin >> u >> v;
        g[u].push_back(v);
        g[v].push_back(u);
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}