#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <stack>
#include <set>

typedef long long llong;
const int MAXN = 100000 + 10;
const int MOD = 1e9 + 7;

int n, d;
struct Value
{
    int l, w;
    void fix()
    {
        if (l >= MOD) l -= MOD;
        if (w >= MOD) w -= MOD;
    }

    friend Value operator + (const Value &left, const Value &right)
    {
        Value result;
        result.l = left.l + right.l;
        result.w = left.w + right.w;
        result.fix();
        return result;
    }

    friend Value operator - (const Value &left, const Value &right)
    {
        Value result;
        result.l = left.l + MOD - right.l;
        result.w = left.w + MOD - right.w;
        result.fix();
        return result;
    }
};

int sz[MAXN];
bool win[MAXN];
int cntLose[MAXN];
Value dpW[MAXN];
Value dpL[MAXN];
Value sumW[MAXN];
Value sumLW[MAXN];
Value coefsL, coefsW;
std::vector <int> g[MAXN];

void dfs(int node, int par)
{
    int cnt = 0;
    int v = 0;

    sz[node] = 1;
    cntLose[node] = 0;
    for (const int &u : g[node])
    {
        if (u == par)
        {
            continue;
        }

        dfs(u, node);
        sz[node] += sz[u];
        win[node] |= !win[u];
        cntLose[node] += !win[u];
        sumW[node] = sumW[node] + dpW[u];
        if (!win[u])
        {
            sumLW[node] = sumLW[node] + dpW[u];
        }

        if (!win[u])
        {
            v = u;
        }
    }

    if (cntLose[node] >= 2)
    {
        dpL[node] = {0, 0};
    } else if (cntLose[node] == 1)
    {
        dpL[node] = dpW[v];
    } else
    {
        dpL[node] = {0, 0};
        dpL[node].w = 1;
        for (const int &u : g[node])
        {
            if (u == par)
            {
                continue;
            }

            dpL[node] = dpL[node] + dpW[u];
        }
    }

    dpW[node].w = sz[node];
    dpW[node].l = sz[node];
    dpW[node] = dpW[node] - dpL[node];
}

void dfsUp(int node, int par)
{
    if (node != 1) 
    {
        bool parentIsUnwinning = false;
        if (!win[node] && cntLose[par] == 1)
        {
            parentIsUnwinning = true;
            win[node] = true;
            cntLose[node] = g[node].size() - 1;
        } else if (!win[par])
        {
            parentIsUnwinning = true;
            cntLose[node]++;
        }
    
        Value parentL;
        if (cntLose[par] == 0)
        {
            parentL = dpL[par] - dpW[node];
        } else if (cntLose[par] == 1 && win[node])
        {
            parentL = dpL[par];
        } else if (cntLose[par] == 1 && !win[node])
        {
            parentL = sumW[par] - dpW[node];
            parentL.w++;
        } else if (cntLose[par] == 2 && !win[node])
        {
            parentL = sumLW[par] - dpW[node];
        } else
        {
            assert(cntLose[par] >= 2);
            parentL = {0, 0};
        }

        Value parentW = {n - sz[node], n - sz[node]};
        parentW = parentW - parentL;

        if (cntLose[node] >= 2)
        {
            dpL[node] = {0, 0};
        } else if (cntLose[node] == 1)
        {
            if (parentIsUnwinning)
            {
                dpL[node] = parentW;
            }
        } else
        {
            dpL[node] = {0, 0};
            dpL[node].w = 1;
            for (const int &u : g[node])
            {
                if (u == par)
                {
                    dpL[node] = dpL[node] + parentW;
                    continue;
                }

                dpL[node] = dpL[node] + dpW[u];
            }
        }

        sumW[node] = parentW;
        for (const int &u : g[node])
        {
            if (u == par)
            {
                continue;
            }

            sumW[node] = sumW[node] + dpW[u];
        }

        sumLW[node] = {0, 0};
        if (parentIsUnwinning)
        {
            sumLW[node] = parentW;
        }

        for (const int &u : g[node])
        {
            if (u == par || win[u])
            {
                continue;
            }

            sumLW[node] = sumLW[node] + dpW[u];
        }

        dpW[node].w = n;
        dpW[node].l = n;
        dpW[node] = dpW[node] - dpL[node];
    }

    for (const int &u : g[node])
    {
        if (u == par)
        {
            continue;
        }

        dfsUp(u, node);
    }
}

void solve()
{   
    dfs(1, 0);
    dfsUp(1, 0);
    // return;

    int l = 0;
    int w = 0;
    for (int i = 1 ; i <= n ; ++i)
    {
        w += win[i];
        l += !win[i];
        coefsL = coefsL + dpL[i];
        coefsW = coefsW + dpW[i];
    }

    for (int i = 1 ; i < d ; ++i)
    {
        int newL, newW;
        newL = (1LL * coefsL.l * l + 1LL * coefsL.w * w) % MOD;
        newW = (1LL * coefsW.l * l + 1LL * coefsW.w * w) % MOD;
        l = newL; w = newW;
    }

    int ans = (1LL * dpW[1].l * l + 1LL * dpW[1].w * w) % MOD;
    std::cout << ans << '\n';
}

void input()
{
    std::cin >> n >> d;
    for (int i = 1 ; i < n ; ++i)
    {
        int u, v;
        std::cin >> u >> v;
        g[u].push_back(v);
        g[v].push_back(u);
    }
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIOI();
    input();
    solve();

    return 0;
}